/* eslint react/prop-types: 0 */
import { each, includes, keys } from 'lodash'

import { Component, documentHelpers } from 'substance'

class NoteComponent extends Component {

  didMount () {
    this.context.editorSession.onUpdate('', this.showNote, this)
  }

  render ($$) {
    const el = $$('note')
      .attr('note-content', this.props.node['note-content'])
      .addClass('sc-note').on('click', this.showNote)

    return el
  }

  showNote () {
    const selected = this.getSelection()
    if (!selected.node) return
    const provider = this.getProvider()
    this.disableTools(selected)
    provider.showNote(selected.node)
  }

  disableTools () {
    const commandStates = this.getCommandStates()
    each(keys(commandStates), (key) => {
      const allowed = ['comment', 'redo', 'save', 'switch-text-type', 'undo', 'note']
      if (!includes(allowed, key)) commandStates[key].disabled = true
    })
    this.rerender()
  }

  getSelection () {
    // TODO -- write cleaner
    const surface = this.getSurface()
    if (!surface) return {}

    const session = this.context.editorSession
    const sel = session.getSelection()

    const notes = documentHelpers.getPropertyAnnotationsForSelection(
      session.getDocument(),
      sel,
      { type: 'note' }
    )
    const note = notes[0]
    let show = false
    if (typeof note !== 'undefined') {
      if ((sel.start.offset === note.start.offset &&
        sel.end.offset === note.end.offset)) {
        show = true
      }
    }

    if (show) {
      return {
        node: note
      }
    } else {
      return {
        node: null
      }
    }
  }

  getSurface () {
    return this.context.surfaceManager.getFocusedSurface()
  }

  getCommandStates () {
    return this.context.commandManager.commandStates
  }

  getProvider () {
    return this.context.notesProvider
  }

  dispose () {
    this.props.node.off(this)
  }
}

export default NoteComponent
